@@ -51,7 +51,7 @@ LfoWidget::LfoWidget(MidiLfo *p_midiWorker, int portCount, bool compactStyle,
     int l1;
     QStringList midiCCNames;
     midiCCNames << "MuteToggle" << "Amplitude" << "Offset" << "WaveForm" << "Frequency"
-                << "RecordToggle"<< "Resolution"<< "Size" << "unknown";
+                << "RecordToggle"<< "Resolution"<< "Size" << "LoopMode" << "unknown";
     midiControl = new MidiControl(midiCCNames);
 
     manageBox = new ManageBox("LFO:", true, this);
@@ -59,6 +59,28 @@ LfoWidget::LfoWidget(MidiLfo *p_midiWorker, int portCount, bool compactStyle,
     // Input group box on right top
     QGroupBox *inBox = new QGroupBox(tr("Input"), this);
 
+    QLabel *enableNoteOffLabel = new QLabel(tr("&Note Off"),inBox);
+    enableNoteOff = new QCheckBox(this);
+    connect(enableNoteOff, SIGNAL(toggled(bool)), this, SLOT(updateEnableNoteOff(bool)));
+    enableNoteOffLabel->setBuddy(enableNoteOff);
+    enableNoteOff->setToolTip(tr("Stop output when Note is released"));
+
+    QLabel *enableRestartByKbdLabel = new QLabel(tr("&Restart"),inBox);
+    enableRestartByKbd = new QCheckBox(this);
+    connect(enableRestartByKbd, SIGNAL(toggled(bool)), this, SLOT(updateEnableRestartByKbd(bool)));
+    enableRestartByKbdLabel->setBuddy(enableRestartByKbd);
+    enableRestartByKbd->setToolTip(tr("Restart sequence when a new note is received"));
+
+    QLabel *enableTrigByKbdLabel = new QLabel(tr("&Trigger"),inBox);
+    enableTrigByKbd = new QCheckBox(this);
+    connect(enableTrigByKbd, SIGNAL(toggled(bool)), this, SLOT(updateEnableTrigByKbd(bool)));
+    enableTrigByKbdLabel->setBuddy(enableTrigByKbd);
+    enableTrigByKbd->setToolTip(tr("Retrigger sequence when a new note is received"));
+
+    enableNoteOff->setChecked(false);
+    enableRestartByKbd->setChecked(false);
+    enableTrigByKbd->setChecked(false);
+
     QLabel *ccnumberInLabel = new QLabel(tr("MIDI &CC#"), inBox);
     ccnumberInBox = new QSpinBox(inBox);
     ccnumberInLabel->setBuddy(ccnumberInBox);
@@ -80,8 +102,14 @@ LfoWidget::LfoWidget(MidiLfo *p_midiWorker, int portCount, bool compactStyle,
 
     inBoxLayout->addWidget(ccnumberInLabel, 0, 0);
     inBoxLayout->addWidget(ccnumberInBox, 0, 1);
-    inBoxLayout->addWidget(chInLabel, 2, 0);
-    inBoxLayout->addWidget(chIn, 2, 1);
+    inBoxLayout->addWidget(enableNoteOffLabel, 1, 0);
+    inBoxLayout->addWidget(enableNoteOff, 1, 1);
+    inBoxLayout->addWidget(enableRestartByKbdLabel, 2, 0);
+    inBoxLayout->addWidget(enableRestartByKbd, 2, 1);
+    inBoxLayout->addWidget(enableTrigByKbdLabel, 3, 0);
+    inBoxLayout->addWidget(enableTrigByKbd, 3, 1);
+    inBoxLayout->addWidget(chInLabel, 4, 0);
+    inBoxLayout->addWidget(chIn, 4, 1);
     if (compactStyle) {
         inBoxLayout->setSpacing(1);
         inBoxLayout->setMargin(2);
@@ -225,6 +253,18 @@ LfoWidget::LfoWidget(MidiLfo *p_midiWorker, int portCount, bool compactStyle,
             SLOT(updateSize(int)));
     midiControl->addMidiLearnMenu(sizeBox, 7);
 
+    loopBox = new QComboBox(waveBox);
+    names.clear();
+    names << "->_>" << " <_<-" << "->_<" << " >_<-" << "->_|" << " |_<-";
+    loopBox->insertItems(0, names);
+    loopBox->setCurrentIndex(0);
+    loopBox->setToolTip(tr("Loop, bounce or play once going forward or backward"));
+    loopBox->setMinimumContentsLength(5);
+    connect(loopBox, SIGNAL(activated(int)), this,
+            SLOT(updateLoop(int)));
+    midiControl->addMidiLearnMenu(loopBox, 8);
+
+
     QLabel *recordButtonLabel = new QLabel(tr("Re&cord"), waveBox);
     recordAction = new QAction(QIcon(seqrecord_xpm), tr("Re&cord"), waveBox);
     recordAction->setToolTip(tr("Record incoming controller"));
@@ -258,8 +298,9 @@ LfoWidget::LfoWidget(MidiLfo *p_midiWorker, int portCount, bool compactStyle,
     }
 
     QGridLayout *paramBoxLayout = new QGridLayout;
-    paramBoxLayout->addWidget(recordButtonLabel, 0, 0);
-    paramBoxLayout->addWidget(recordButton, 0, 1);
+    paramBoxLayout->addWidget(loopBox, 0, 0, 1, 2);
+    paramBoxLayout->addWidget(recordButtonLabel, 1, 0);
+    paramBoxLayout->addWidget(recordButton, 1, 1);
     paramBoxLayout->addWidget(waveFormBoxLabel, 0, 2);
     paramBoxLayout->addWidget(waveFormBox, 0, 3);
     paramBoxLayout->addWidget(freqBoxLabel, 1, 2);
@@ -314,6 +355,12 @@ void LfoWidget::writeData(QXmlStreamWriter& xml)
     xml.writeStartElement(manageBox->name.left(3));
     xml.writeAttribute("name", manageBox->name.mid(manageBox->name.indexOf(':') + 1));
         xml.writeStartElement("input");
+            xml.writeTextElement("enableNoteOff", QString::number(
+                midiWorker->enableNoteOff));
+            xml.writeTextElement("restartByKbd", QString::number(
+                midiWorker->restartByKbd));
+            xml.writeTextElement("trigByKbd", QString::number(
+                midiWorker->trigByKbd));
             xml.writeTextElement("channel", QString::number(
                 midiWorker->chIn));
             xml.writeTextElement("ccnumber", QString::number(
@@ -332,6 +379,8 @@ void LfoWidget::writeData(QXmlStreamWriter& xml)
         xml.writeEndElement();
 
         xml.writeStartElement("waveParams");
+            xml.writeTextElement("loopmode", QString::number(
+                loopBox->currentIndex()));
             xml.writeTextElement("waveform", QString::number(
                 waveFormBox->currentIndex()));
             xml.writeTextElement("frequency", QString::number(
@@ -387,6 +436,12 @@ void LfoWidget::readData(QXmlStreamReader& xml)
                 xml.readNext();
                 if (xml.isEndElement())
                     break;
+                if (xml.name() == "enableNoteOff")
+                    enableNoteOff->setChecked(xml.readElementText().toInt());
+                else if (xml.name() == "restartByKbd")
+                    enableRestartByKbd->setChecked(xml.readElementText().toInt());
+                else if (xml.name() == "trigByKbd")
+                    enableTrigByKbd->setChecked(xml.readElementText().toInt());
                 if (xml.name() == "channel") {
                     tmp = xml.readElementText().toInt();
                     chIn->setCurrentIndex(tmp);
@@ -426,7 +481,12 @@ void LfoWidget::readData(QXmlStreamReader& xml)
                 xml.readNext();
                 if (xml.isEndElement())
                     break;
-                if (xml.name() == "waveform")
+                if (xml.name() == "loopmode") {
+                    tmp = xml.readElementText().toInt();
+                    loopBox->setCurrentIndex(tmp);
+                    updateLoop(tmp);
+                }
+                else if (xml.name() == "waveform")
                     wvtmp = xml.readElementText().toInt();
                 else if (xml.name() == "frequency") {
                     tmp = xml.readElementText().toInt();
@@ -515,91 +575,6 @@ void LfoWidget::skipXmlElement(QXmlStreamReader& xml)
     }
 }
 
-void LfoWidget::readDataText(QTextStream& arpText)
-{
-    QString qs, qs2;
-    int l1, lt, wvtmp;
-    Sample sample;
-
-    qs = arpText.readLine();
-    qs2 = qs.section(' ', 0, 0);
-    channelOut->setCurrentIndex(qs2.toInt());
-    qs2 = qs.section(' ', 1, 1);
-    portOut->setCurrentIndex(qs2.toInt());
-    qs2 = qs.section(' ', 2, 2);
-    ccnumberBox->setValue(qs2.toInt());
-    qs = arpText.readLine();
-    qs2 = qs.section(' ', 0, 0);
-    freqBox->setCurrentIndex(qs2.toInt());
-    updateFreq(qs2.toInt());
-    qs2 = qs.section(' ', 1, 1);
-    resBox->setCurrentIndex(qs2.toInt());
-    updateRes(qs2.toInt());
-    qs2 = qs.section(' ', 2, 2);
-    sizeBox->setCurrentIndex(qs2.toInt());
-    updateSize(qs2.toInt());
-    qs2 = qs.section(' ', 3, 3);
-    amplitude->setValue(qs2.toInt());
-    qs2 = qs.section(' ', 4, 4);
-    offset->setValue(qs2.toInt());
-    qs = arpText.readLine();
-    if (qs == "MIDICC")
-    {
-        qs = arpText.readLine();
-        while (qs != "EOCC") {
-            qs2 = qs.section(' ', 0, 0);
-            int controlID = qs2.toInt();
-            qs2 = qs.section(' ', 1, 1);
-            int ccnumber = qs2.toInt();
-            qs2 = qs.section(' ', 2, 2);
-            int channel = qs2.toInt();
-            qs2 = qs.section(' ', 3, 3);
-            int min = qs2.toInt();
-            qs2 = qs.section(' ', 4, 4);
-            int max = qs2.toInt();
-            midiControl->appendMidiCC(controlID, ccnumber, channel, min, max);
-            qs = arpText.readLine();
-        }
-    qs = arpText.readLine();
-    }
-
-    wvtmp = qs.toInt();
-
-    // Read Mute Mask
-    int step = TPQN / midiWorker->res;
-    qs = arpText.readLine();
-    if (qs.isEmpty() || (qs == "EOP")) return;
-    qs2 = qs.section(' ', 0, 0);
-    midiWorker->muteMask.clear();
-    l1 = 0;
-    while (qs2 !="EOM") {
-        midiWorker->muteMask.append(qs2.toInt());
-        l1++;
-        if (!(l1%32)) qs = arpText.readLine();
-        qs2 = qs.section(' ', l1%32, l1%32);
-    }
-
-    // Read Custom Waveform
-    qs = arpText.readLine();
-    qs2 = qs.section(' ', 0, 0);
-    midiWorker->customWave.clear();
-    l1 = 0;
-    lt = 0;
-    while (qs2 !="EOW") {
-        sample.value=qs2.toInt();
-        sample.tick = lt;
-        sample.muted = midiWorker->muteMask.at(l1);
-        midiWorker->customWave.append(sample);
-        lt+=step;
-        l1++;
-        if (!(l1%16)) qs = arpText.readLine();
-        qs2 = qs.section(' ', l1%16, l1%16);
-    }
-    waveFormBox->setCurrentIndex(wvtmp);
-    updateWaveForm(wvtmp);
-    modified = false;
-}
-
 void LfoWidget::loadWaveForms()
 {
     waveForms << tr("Sine") << tr("Saw up") << tr("Triangle")
@@ -629,6 +604,23 @@ void LfoWidget::updateCcnumberIn(int val)
     midiWorker->ccnumberIn = val;
     modified = true;
 }
+void LfoWidget::updateEnableNoteOff(bool on)
+{
+    midiWorker->enableNoteOff = on;
+    modified = true;
+}
+
+void LfoWidget::updateEnableRestartByKbd(bool on)
+{
+    midiWorker->restartByKbd = on;
+    modified = true;
+}
+
+void LfoWidget::updateEnableTrigByKbd(bool on)
+{
+    midiWorker->trigByKbd = on;
+    modified = true;
+}
 
 void LfoWidget::updateWaveForm(int val)
 {
@@ -681,6 +673,13 @@ void LfoWidget::updateSize(int val)
     modified = true;
 }
 
+void LfoWidget::updateLoop(int val)
+{
+    if (val > 5) return;
+    midiWorker->updateLoop(val);
+    modified = true;
+}
+
 void LfoWidget::updateAmp(int val)
 {
     midiWorker->updateAmplitude(val);
@@ -814,6 +813,10 @@ void LfoWidget::copyParamsFrom(LfoWidget *fromWidget)
 {
     int tmp;
 
+    enableNoteOff->setChecked(fromWidget->enableNoteOff->isChecked());
+    enableRestartByKbd->setChecked(fromWidget->enableRestartByKbd->isChecked());
+    enableTrigByKbd->setChecked(fromWidget->enableTrigByKbd->isChecked());
+
     tmp = fromWidget->chIn->currentIndex();
     chIn->setCurrentIndex(tmp);
     updateChIn(tmp);
@@ -837,6 +840,9 @@ void LfoWidget::copyParamsFrom(LfoWidget *fromWidget)
     tmp = fromWidget->sizeBox->currentIndex();
     sizeBox->setCurrentIndex(tmp);
     updateSize(tmp);
+    tmp = fromWidget->loopBox->currentIndex();
+    loopBox->setCurrentIndex(tmp);
+    updateLoop(tmp);
     tmp = fromWidget->freqBox->currentIndex();
     freqBox->setCurrentIndex(tmp);
     updateFreq(tmp);
@@ -861,3 +867,88 @@ QVector<Sample> LfoWidget::getCustomWave()
 {
     return midiWorker->customWave;
 }
+
+void LfoWidget::handleController(int ccnumber, int channel, int value)
+{
+    bool m;
+    int min, max, sval;
+    QVector<MidiCC> cclist= midiControl->ccList;
+
+    for (int l2 = 0; l2 < cclist.count(); l2++) {
+        min = cclist.at(l2).min;
+        max = cclist.at(l2).max;
+        if ((ccnumber == cclist.at(l2).ccnumber) &&
+            (channel == cclist.at(l2).channel)) {
+            switch (cclist.at(l2).ID) {
+                case 0: if (min == max) {
+                            if (value == max) {
+                                m = muteOut->isChecked();
+                                muteOut->setChecked(!m);
+                            }
+                        }
+                        else {
+                            if (value == max) {
+                                muteOut->setChecked(false);
+                            }
+                            if (value == min) {
+                                muteOut->setChecked(true);
+                            }
+                        }
+                break;
+
+                case 1:
+                        sval = min + ((double)value * (max - min) / 127);
+                        amplitude->setValue(sval);
+                break;
+
+                case 2:
+                        sval = min + ((double)value * (max - min) / 127);
+                        offset->setValue(sval);
+                break;
+                case 3:
+                        sval = min + ((double)value * (max - min) / 127);
+                        waveFormBox->setCurrentIndex(sval);
+                        updateWaveForm(sval);
+                break;
+                case 4:
+                        sval = min + ((double)value * (max - min) / 127);
+                        freqBox->setCurrentIndex(sval);
+                        updateFreq(sval);
+                break;
+                case 5: if (min == max) {
+                            if (value == max) {
+                                m = recordAction->isChecked();
+                                recordAction->setChecked(!m);
+                            }
+                        }
+                        else {
+                            if (value == max) {
+                                recordAction->setChecked(true);
+                            }
+                            if (value == min) {
+                                recordAction->setChecked(false);
+                            }
+                        }
+                break;
+                case 6:
+                        sval = min + ((double)value * (max - min) / 127);
+                        resBox->setCurrentIndex(sval);
+                        updateRes(sval);
+                break;
+                case 7:
+                        sval = min + ((double)value * (max - min) / 127);
+                        sizeBox->setCurrentIndex(sval);
+                        updateSize(sval);
+                break;
+                case 8:
+                        sval = min + ((double)value * (max - min) / 127);
+                        loopBox->setCurrentIndex(sval);
+                        updateLoop(sval);
+                break;
+
+                default:
+                break;
+            }
+        }
+    }
+}
