@@ -69,22 +69,24 @@ static const char FILEEXT[] = ".qmax";
 
 int MainWindow::sigpipe[2];
 
-MainWindow::MainWindow(int p_portCount)
+MainWindow::MainWindow(int p_portCount, bool p_alsamidi)
 {
     filename = "";
     lastDir = QDir::homePath();
+    alsaMidi = p_alsamidi;
 
     grooveWidget = new GrooveWidget(this);
     grooveWindow = new QDockWidget(tr("Groove"), this);
     grooveWindow->setFeatures(QDockWidget::DockWidgetClosable
             | QDockWidget::DockWidgetMovable
             | QDockWidget::DockWidgetFloatable);
-    grooveWindow->setWidget(grooveWidget);;
+    grooveWindow->setWidget(grooveWidget);
     grooveWindow->setObjectName("grooveWidget");
     grooveWindow->setVisible(true);
     addDockWidget(Qt::BottomDockWidgetArea, grooveWindow);
 
-    engine = new Engine(grooveWidget, p_portCount, this);
+    engine = new Engine(grooveWidget, p_portCount, alsaMidi, this);
+    if (!alsaMidi) connect(engine->driver, SIGNAL(jsEvent(int)), this, SLOT(jsAction(int)));
 
     midiCCTable = new MidiCCTable(engine, this);
 
@@ -97,17 +99,14 @@ MainWindow::MainWindow(int p_portCount)
     logWindow->setObjectName("logWidget");
     addDockWidget(Qt::BottomDockWidgetArea, logWindow);
     qRegisterMetaType<MidiEvent>("MidiEvent");
-    connect(engine->seqDriver, SIGNAL(midiEvent(MidiEvent)),
-            logWidget, SLOT(appendEvent(MidiEvent)));
+    connect(engine, SIGNAL(midiEventReceived(MidiEvent, int)),
+            logWidget, SLOT(appendEvent(MidiEvent, int)));
 
     connect(logWidget, SIGNAL(sendLogEvents(bool)),
-            engine->seqDriver, SLOT(setSendLogEvents(bool)));
+            engine, SLOT(setSendLogEvents(bool)));
 
     passWidget = new PassWidget(engine, p_portCount, this);
 
-    connect(this, SIGNAL(runQueue(bool)),
-            engine->seqDriver, SLOT(setQueueStatus(bool)));
-
     addArpAction = new QAction(QIcon(arpadd_xpm), tr("&New Arp..."), this);
     addArpAction->setShortcut(QKeySequence(tr("Ctrl+A", "Module|New Arp")));
     addArpAction->setToolTip(tr("Add new arpeggiator to tab bar"));
@@ -151,7 +150,7 @@ MainWindow::MainWindow(int p_portCount)
     connect(fileQuitAction, SIGNAL(triggered()), this, SLOT(close()));
 
     runAction = new QAction(QIcon(play_xpm), tr("&Run with internal clock"), this);
-    connect(runAction, SIGNAL(toggled(bool)), this, SLOT(updateRunQueue(bool)));
+    connect(runAction, SIGNAL(toggled(bool)), this, SLOT(updateTransportStatus(bool)));
     runAction->setCheckable(true);
     runAction->setChecked(false);
     runAction->setDisabled(true);
@@ -176,15 +175,20 @@ MainWindow::MainWindow(int p_portCount)
     jackSyncAction = new QAction(QIcon(jacktr_xpm),
             tr("&Connect to Jack Transport"), this);
     jackSyncAction->setCheckable(true);
-    jackSyncAction->setChecked(false);
-    jackSyncAction->setDisabled(true);
     connect(jackSyncAction, SIGNAL(toggled(bool)), this,
             SLOT(jackSyncToggle(bool)));
-    connect(engine->seqDriver, SIGNAL(jackShutdown(bool)),
-            jackSyncAction, SLOT(setChecked(bool)));
+    if (alsaMidi) {
+        connect(engine->jackSync, SIGNAL(j_shutdown()), this,
+                SLOT(jackShutdown()));
+    }
+    else {
+        connect(engine->driver, SIGNAL(j_shutdown()), this,
+                SLOT(jackShutdown()));
+    }
+    jackSyncAction->setChecked(!alsaMidi);
+    jackSyncAction->setDisabled(true);
 
-
-    updateRunQueue(false);
+    updateTransportStatus(false);
 
     QAction* viewLogAction = logWindow->toggleViewAction();
     viewLogAction->setIcon(QIcon(eventlog_xpm));
@@ -298,12 +302,12 @@ void MainWindow::updateWindowTitle()
     if (filename.isEmpty())
         setWindowTitle(QString("%1 (%2)")
                 .arg(APP_NAME)
-                .arg(engine->getAlsaClientId()));
+                .arg(engine->getClientId()));
     else
         setWindowTitle(QString("%1 - %2  (%3)")
                 .arg(filename)
                 .arg(APP_NAME)
-                .arg(engine->getAlsaClientId()));
+                .arg(engine->getClientId()));
 }
 
 void MainWindow::helpAbout()
@@ -359,7 +363,6 @@ void MainWindow::addArp(const QString& name)
 {
     int count, widgetID;
     MidiArp *midiWorker = new MidiArp();
-    engine->addMidiArp(midiWorker);
     ArpWidget *moduleWidget = new ArpWidget(midiWorker,
             engine->getPortCount(), passWidget->compactStyle,
             passWidget->mutedAdd, this);
@@ -376,18 +379,12 @@ void MainWindow::addArp(const QString& name)
     connect(moduleWidget->midiControl, SIGNAL(setMidiLearn(int, int, int)),
             engine, SLOT(setMidiLearn(int, int, int)));
 
-    connect(grooveWidget, SIGNAL(newGrooveTick(int)),
-            moduleWidget->screen, SLOT(setGrooveTick(int)));
-    connect(grooveWidget, SIGNAL(newGrooveVelocity(int)),
-            moduleWidget->screen, SLOT(setGrooveVelocity(int)));
-    connect(grooveWidget, SIGNAL(newGrooveLength(int)),
-            moduleWidget->screen, SLOT(setGrooveLength(int)));
-
     widgetID = engine->arpWidgetCount();
     moduleWidget->manageBox->name = name;
     moduleWidget->manageBox->ID = widgetID;
     moduleWidget->midiControl->ID = widgetID;
 
+    engine->addMidiArp(midiWorker);
     engine->addArpWidget(moduleWidget);
     engine->sendGroove();
 
@@ -403,7 +400,6 @@ void MainWindow::addLfo(const QString& name)
 {
     int widgetID, count;
     MidiLfo *midiWorker = new MidiLfo();
-    engine->addMidiLfo(midiWorker);
     LfoWidget *moduleWidget = new LfoWidget(midiWorker,
             engine->getPortCount(), passWidget->compactStyle,
             passWidget->mutedAdd, this);
@@ -422,6 +418,7 @@ void MainWindow::addLfo(const QString& name)
     moduleWidget->manageBox->ID = widgetID;
     moduleWidget->midiControl->ID = widgetID;
 
+    engine->addMidiLfo(midiWorker);
     engine->addLfoWidget(moduleWidget);
     count = engine->moduleWindowCount();
     moduleWidget->manageBox->parentDockID = count;
@@ -435,7 +432,6 @@ void MainWindow::addSeq(const QString& name)
 {
     int widgetID, count;
     MidiSeq *midiWorker = new MidiSeq();
-    engine->addMidiSeq(midiWorker);
     SeqWidget *moduleWidget = new SeqWidget(midiWorker,
             engine->getPortCount(), passWidget->compactStyle,
             passWidget->mutedAdd, this);
@@ -455,6 +451,7 @@ void MainWindow::addSeq(const QString& name)
     moduleWidget->manageBox->ID = widgetID;
     moduleWidget->midiControl->ID = widgetID;
 
+    engine->addMidiSeq(midiWorker);
     engine->addSeqWidget(moduleWidget);
     count = engine->moduleWindowCount();
     moduleWidget->manageBox->parentDockID = count;
@@ -468,7 +465,6 @@ void MainWindow::cloneLfo(int ID)
 {
     int widgetID, count;
     MidiLfo *midiWorker = new MidiLfo();
-    engine->addMidiLfo(midiWorker);
     LfoWidget *moduleWidget = new LfoWidget(midiWorker,
             engine->getPortCount(), passWidget->compactStyle,
             passWidget->mutedAdd, this);
@@ -489,13 +485,15 @@ void MainWindow::cloneLfo(int ID)
 
     moduleWidget->copyParamsFrom(engine->lfoWidget(ID));
 
+    midiWorker->setFramePtr(engine->lfoWidget(ID)->getFramePtr());
+    midiWorker->nextTick = engine->lfoWidget(ID)->getNextTick();
+    engine->addMidiLfo(midiWorker);
     engine->addLfoWidget(moduleWidget);
     count = engine->moduleWindowCount();
     moduleWidget->manageBox->parentDockID = count;
     moduleWidget->midiControl->parentDockID = count;
     appendDock(moduleWidget, moduleWidget->manageBox->name, count);
 
-    checkIfFirstModule();
 }
 
 void MainWindow::cloneSeq(int ID)
@@ -504,7 +502,6 @@ void MainWindow::cloneSeq(int ID)
     QString name;
     MidiSeq *midiWorker = new MidiSeq();
 
-    engine->addMidiSeq(midiWorker);
     SeqWidget *moduleWidget = new SeqWidget(midiWorker,
             engine->getPortCount(), passWidget->compactStyle,
             passWidget->mutedAdd, this);
@@ -526,11 +523,15 @@ void MainWindow::cloneSeq(int ID)
 
     moduleWidget->copyParamsFrom(engine->seqWidget(ID));
 
+    midiWorker->setCurrentIndex(engine->seqWidget(ID)->getCurrentIndex());
+    midiWorker->nextTick = engine->seqWidget(ID)->getNextTick();
+    engine->addMidiSeq(midiWorker);
     engine->addSeqWidget(moduleWidget);
     count = engine->moduleWindowCount();
     moduleWidget->manageBox->parentDockID = count;
     moduleWidget->midiControl->parentDockID = count;
     appendDock(moduleWidget, moduleWidget->manageBox->name, count);
+
 }
 
 void MainWindow::appendDock(QWidget *moduleWidget, const QString &name, int count)
@@ -603,8 +604,8 @@ void MainWindow::removeSeq(int index)
 
 void MainWindow::clear()
 {
-    updateRunQueue(false);
-    jackSyncToggle(false);
+    updateTransportStatus(false);
+    if (alsaMidi) jackSyncToggle(false);
 
     while (engine->midiArpCount()) {
         removeArp(engine->midiArpCount() - 1);
@@ -617,6 +618,8 @@ void MainWindow::clear()
     while (engine->midiSeqCount()) {
         removeSeq(engine->midiSeqCount() - 1);
     }
+
+    grooveWidget->midiControl->ccList.clear();
 }
 
 void MainWindow::fileNew()
@@ -639,15 +642,11 @@ void MainWindow::chooseFile()
 {
     QString fn =  QFileDialog::getOpenFileName(this,
             tr("Open arpeggiator file"), lastDir,
-            tr("QMidiArp XML files")  + " (*" + FILEEXT + ");;"
-            + tr("Old QMidiArp files") + " (*.qma)");
+            tr("QMidiArp XML files")  + " (*" + FILEEXT + ")");
     if (fn.isEmpty())
         return;
 
-    if (fn.endsWith(".qma"))
-        openTextFile(fn);
-
-    else if (fn.endsWith(FILEEXT))
+    if (fn.endsWith(FILEEXT))
         openFile(fn);
 }
 
@@ -719,10 +718,14 @@ void MainWindow::readFilePartGlobal(QXmlStreamReader& xml)
                     break;
                 if (xml.name() == "midiControlEnabled")
                     passWidget->cbuttonCheck->setChecked(xml.readElementText().toInt());
-                else if (xml.name() == "midiClockEnabled")
-                        midiClockAction->setChecked(xml.readElementText().toInt());
-                else if (xml.name() == "jackSyncEnabled")
-                        jackSyncAction->setChecked(xml.readElementText().toInt());
+                else if (xml.name() == "midiClockEnabled") {
+                        bool tmp = xml.readElementText().toInt();
+                        if (alsaMidi) midiClockAction->setChecked(tmp);
+                    }
+                else if (xml.name() == "jackSyncEnabled") {
+                        bool tmp = xml.readElementText().toInt();
+                        if (alsaMidi) jackSyncAction->setChecked(tmp);
+                    }
                 else if (xml.name() == "forwardUnmatched")
                     passWidget->setForward(xml.readElementText().toInt());
                 else if (xml.name() == "forwardPort")
@@ -807,110 +810,6 @@ void MainWindow::skipXmlElement(QXmlStreamReader& xml)
     }
 }
 
-void MainWindow::openTextFile(const QString& fn)
-{
-    QString line, qs, qs2;
-    bool midiclocktmp = false;
-    int c = 0;
-
-    lastDir = fn.left(fn.lastIndexOf('/'));
-
-    QFile f(fn);
-    if (!f.open(QIODevice::ReadOnly)) {
-        QMessageBox::warning(this, APP_NAME,
-                tr("Could not read from file '%1'.").arg(fn));
-        return;
-    }
-
-    clear();
-    filename = fn;
-
-    QTextStream loadText(&f);
-    qs = loadText.readLine();
-    if (qs == "Tempo")
-    {
-        qs = loadText.readLine();
-        tempoSpin->setValue(qs.toInt());
-        qs = loadText.readLine();
-    }
-    if (qs == "MIDI Control")
-    {
-        qs = loadText.readLine();
-        qs2 = qs.section(' ', 0, 0);
-        passWidget->cbuttonCheck->setChecked(qs2.toInt());
-        qs = loadText.readLine();
-    }
-    if (qs == "MIDI Clock")
-    {
-        qs = loadText.readLine();
-        qs2 = qs.section(' ', 0, 0);
-        midiclocktmp = qs2.toInt();
-        qs = loadText.readLine();
-    }
-    qs2 = qs.section(' ', 0, 0);
-    passWidget->setForward(qs2.toInt());
-    qs2 = qs.section(' ', 1, 1);
-    passWidget->setPortUnmatched(qs2.toInt() + 1);
-    qs = loadText.readLine();
-    qs2 = qs.section(' ', 0, 0);
-
-    grooveWidget->grooveTick->setValue(qs2.toInt());
-    //  engine->seqDriver->setGrooveTick(qs2.toInt());
-    qs2 = qs.section(' ', 1, 1);
-    grooveWidget->grooveVelocity->setValue(qs2.toInt());
-    //  engine->seqDriver->setGrooveVelocity(qs2.toInt());
-    qs2 = qs.section(' ', 2, 2);
-    grooveWidget->grooveLength->setValue(qs2.toInt());
-    //  engine->seqDriver->setGrooveLength(qs2.toInt());
-
-    while (!loadText.atEnd()) {
-        qs = loadText.readLine();
-        if (qs.startsWith("GUI"))
-            break;
-        if (qs.startsWith("Seq:"))
-            c = 1;
-        if (qs.startsWith("LFO:"))
-            c = 2;
-        if (qs.startsWith("Arp:"))
-            c = 3;
-
-        switch (c) {
-            case 1:
-                addSeq(qs);
-                engine->seqWidget(engine->midiSeqCount() - 1)->readDataText(loadText);
-            break;
-            case 2:
-                addLfo(qs);
-                engine->lfoWidget(engine->midiLfoCount() - 1)->readDataText(loadText);
-            break;
-            case 3:
-                addArp(qs);
-                engine->arpWidget(engine->midiArpCount() - 1)->readDataText(loadText);
-            break;
-            default:
-                qs = "Arp: " + qs;
-                addArp(qs);
-                engine->arpWidget(engine->midiArpCount() - 1)->readDataText(loadText);
-            break;
-        }
-    }
-
-    if (qs.startsWith("GUI")) {
-        qs = loadText.readLine();
-        QByteArray array = QByteArray::fromHex(qs.toLatin1());
-        restoreState(array);
-    }
-
-    midiClockAction->setChecked(midiclocktmp);
-
-    filename.append("x");
-    QMessageBox::warning(this, APP_NAME,
-            tr("The QMidiArp text file was imported. If you save this file, \
-it will be saved using the newer xml format under the name\n '%1'.").arg(filename));
-
-    updateWindowTitle();
-}
-
 void MainWindow::fileSave()
 {
     if (filename.isEmpty())
@@ -951,13 +850,13 @@ bool MainWindow::saveFile()
             xml.writeTextElement("midiControlEnabled",
                 QString::number((int)passWidget->cbuttonCheck->isChecked()));
             xml.writeTextElement("midiClockEnabled",
-                QString::number((int)engine->seqDriver->use_midiclock));
+                QString::number((int)engine->driver->useMidiClock));
             xml.writeTextElement("jackSyncEnabled",
-                QString::number((int)engine->seqDriver->use_jacksync));
+                QString::number((int)engine->driver->useJackSync));
             xml.writeTextElement("forwardUnmatched",
-                QString::number((int)engine->seqDriver->forwardUnmatched));
+                QString::number((int)engine->driver->forwardUnmatched));
             xml.writeTextElement("forwardPort",
-                QString::number(engine->seqDriver->portUnmatched));
+                QString::number(engine->driver->portUnmatched));
         xml.writeEndElement();
 
         xml.writeStartElement("groove");
@@ -1085,25 +984,24 @@ bool MainWindow::isModified()
 
 void MainWindow::updateTempo(int p_tempo)
 {
-    engine->seqDriver->setQueueTempo(p_tempo);
-    engine->setModified(true);
+    engine->setTempo(p_tempo);
 }
 
-void MainWindow::updateRunQueue(bool on)
+void MainWindow::updateTransportStatus(bool on)
 {
-    emit(runQueue(on));
+    engine->setStatus(on);
     tempoSpin->setDisabled(on);
 }
 
-void MainWindow::resetQueue()
+void MainWindow::resetTransport()
 {
-    engine->seqDriver->setQueueStatus(engine->seqDriver->runArp);
+    engine->setStatus(engine->status);
 }
 
 void MainWindow::midiClockToggle(bool on)
 {
-    if (on) jackSyncAction->setChecked(false);
-    engine->seqDriver->setUseMidiClock(on);
+    if (on && alsaMidi) jackSyncAction->setChecked(false);
+    engine->setUseMidiClock(on);
     setGUIforExtSync(on);
 }
 
@@ -1111,18 +1009,27 @@ void MainWindow::jackSyncToggle(bool on)
 {
     if (on) midiClockAction->setChecked(false);
     setGUIforExtSync(on);
-    engine->seqDriver->setUseJackTransport(on);
+    engine->driver->setUseJackTransport(on);
+}
+
+void MainWindow::jackShutdown()
+{
+    engine->setStatus(false);
+    jackSyncAction->setChecked(false);
+    jackSyncAction->setDisabled(true);
 }
 
 void MainWindow::setGUIforExtSync(bool on)
 {
     runAction->setDisabled(on);
     tempoSpin->setDisabled(on);
+    /*
     addArpAction->setDisabled(on);
     addLfoAction->setDisabled(on);
     addSeqAction->setDisabled(on);
     fileOpenAction->setDisabled(on);
     fileRecentlyOpenedFiles->setDisabled(on);
+    * */
 }
 
 bool MainWindow::checkRcFile()
@@ -1323,15 +1230,17 @@ void MainWindow::checkIfLastModule()
         midiClockAction->setDisabled(true);
         midiClockAction->setChecked(false);
         jackSyncAction->setDisabled(true);
-        jackSyncAction->setChecked(false);
+        if (alsaMidi) jackSyncAction->setChecked(false);
     }
 }
 
 void MainWindow::checkIfFirstModule()
 {
     if (engine->moduleWindowCount() == 1) {
-        midiClockAction->setEnabled(true);
-        jackSyncAction->setEnabled(true);
+        if (alsaMidi) {
+            midiClockAction->setEnabled(true);
+            jackSyncAction->setEnabled(true);
+        }
         runAction->setEnabled(!(midiClockAction->isChecked()
                                 || jackSyncAction->isChecked()));
     }
@@ -1406,6 +1315,22 @@ void MainWindow::signalAction(int fd)
     }
 }
 
+void MainWindow::jsAction(int evtype)
+{
+    if (!evtype) {
+        filename = engine->driver->jsFilename;
+        qWarning("JACK Session request to save");
+        lastDir = filename.left(filename.lastIndexOf('/'));
+        updateWindowTitle();
+        bool result = saveFile();
+        if (!result) qWarning("Warning: JACK Session File save failed");
+    }
+    else if (evtype == 1)
+    {
+        close();
+    }
+}
+
 void MainWindow::ctb_update_orientation(Qt::Orientation orient)
 {
     if (orient == Qt::Vertical) {
