@@ -27,9 +27,62 @@
 #include <QString>
 #include "midiarp.h"
 
+/*
+#include "lockfree.h"
+
+// this class must be implemented in such way
+// so that its copy (= operator) is realtime safe
+class MidiEngine
+{
+public:
+    int foo;
+};
+*/
 
 MidiArp::MidiArp()
 {
+    /*
+    // this simulates access from the two threads
+    // lock congestion is simulated by attempt to update
+    // when LockedData destructor is not called
+    {
+        LockFreeStore<MidiEngine> store; // store for the two instances of MidiEngine
+        const MidiEngine * rdata(store); // pointer to the read-only store, to be used from the seq/jack driver thread
+
+        // write to the writable store
+        {
+            LockedData<MidiEngine> ldata(store); // lock the writable store, destructor will unlock it
+            ldata->foo = 1;                       // write access the writable store (lock obtained)
+            qWarning("gui: %d", ldata->foo);      // read access to the writable store (lock obtained)
+        } // ldata gets out of the scope and its destructor is called. this results in unlock of the writable store
+
+        // attempt update the read-only store and report whether the update succeeded
+        // should succeed because the writable store is not locked
+        qWarning("update %s", store.try_lockfree_update() ? "complete" : "not complete (gui thread locked)");
+        // print the data in the read-only store
+        qWarning("seq: %d", rdata->foo);
+
+        // write to the writable store
+        {
+            LockedData<MidiEngine> ldata(store);
+            ldata->foo = 2;
+            qWarning("gui: %d", ldata->foo);
+
+            // attempt update the read-only store and report whether the update succeeded
+            // should fail because the writable store is locked
+            qWarning("update %s", store.try_lockfree_update() ? "complete" : "not complete (gui thread locked)");
+            qWarning("seq: %d (data locked by gui)", rdata->foo);
+        }
+
+        // print the data in the read-only store before the update
+        qWarning("seq: %d (before update)", rdata->foo);
+        // attempt update the read-only store and report whether the update succeeded
+        // should succeed because the writable store is not locked
+        qWarning("update %s", store.try_lockfree_update() ? "complete" : "not complete (gui thread locked)");
+        // print the data in the read-only store after the update
+        qWarning("seq: %d (after update)", rdata->foo);
+    }
+*/
     for (int l1 = 0; l1 < 2; l1++) {
         rangeIn[l1] = (l1) ? 127 : 0;
         indexIn[l1] = (l1) ? 127 : 0;
@@ -39,8 +92,11 @@ MidiArp::MidiArp()
     channelOut = 0;
     noteBufPtr = 0;
     noteCount = 0;
+    releaseNoteCount = 0;
     restartByKbd = false;
     trigByKbd = false;
+    gotKbdTrig = false;
+    restartFlag = false;
     stepWidth = 1.0;     // stepWidth relative to global queue stepWidth
     len = 0.5;       // note length
     vel = 0.8;  // velocity relative to global velocity
@@ -52,7 +108,7 @@ MidiArp::MidiArp()
     newCurrent = false;
     newNext = false;
     currentNoteTick = 0;
-    nextNoteTick = 0;
+    nextTick = 0;
     patternMaxIndex = 0;
     noteOfs = 0;
     arpTick = 0;
@@ -64,13 +120,13 @@ MidiArp::MidiArp()
     randomVelocityAmp = 0;
     randomLengthAmp = 0;
     grooveTick = 0;
+    newGrooveTick = 0;
     grooveVelocity = 0;
     grooveLength = 0;
     repeatPatternThroughChord = 1;
     isMuted = false;
     attack_time = 0.0;
     release_time = 0.0;
-    queueTempo = 100.0;
     sustain = false;
     sustainBuffer.clear();
     latch_mode = false;
@@ -81,7 +137,6 @@ MidiArp::MidiArp()
 }
 
 MidiArp::~MidiArp(){
-    wait();
 }
 
 void MidiArp::setMuted(bool on)
@@ -89,97 +144,108 @@ void MidiArp::setMuted(bool on)
     isMuted = on;
 }
 
-bool MidiArp::wantEvent(MidiEvent event) {
-
-    if (event.channel != chIn) return(false);
-    if ((event.type == EV_CONTROLLER) && (event.data != CT_FOOTSW)) return(false);
-    if (event.type == EV_NOTEON) {
-        if (((event.data < indexIn[0]) || (event.data > indexIn[1]))
-            || ((event.value < rangeIn[0]) || (event.value > rangeIn[1]))) {
-            return(false);
-        }
-    }
-    return(true);
-}
-
-void MidiArp::handleNoteOn(int note, int velocity, int tick)
+bool MidiArp::handleEvent(MidiEvent inEv, int tick, int keep_rel)
 {
-    mutex.lock();
     int bufPtr, index;
 
-    if (!getPressedNoteCount()) {
-        purgeLatchBuffer();
-        if (restartByKbd) advancePatternIndex(true);
-        if (trigByKbd) initArpTick(tick);
+    if (inEv.channel != chIn) return(true);
+    if ((inEv.type == EV_CONTROLLER) && (inEv.data == CT_FOOTSW)) {
+        setSustain((inEv.value == 127), tick);
+        return(false);
     }
-    // modify buffer that is not accessed by arpeggio output
-    bufPtr = (noteBufPtr) ? 0 : 1;
 
-    if (!noteCount || (note > notes[bufPtr][0][noteCount - 1]))
-        index = noteCount;
-    else {
-        index = 0;
-        while (note > notes[bufPtr][0][index]) index++;
+    if (inEv.type != EV_NOTEON) return(true);
+    if (((inEv.data < indexIn[0]) || (inEv.data > indexIn[1]))
+        || ((inEv.value < rangeIn[0]) || (inEv.value > rangeIn[1]))) {
+        return(false);
+    }
+
+    if (inEv.value) {
+        // This is a NOTE ON event
+        if (!getPressedNoteCount()) {
+            purgeLatchBuffer();
+            if (restartByKbd) restartFlag = true;
+            if (trigByKbd) {
+                initArpTick(tick);
+                // if we have been triggered, remove pending release notes
+                if (release_time > 0) {
+                    for (int l1 = 0; l1 < noteCount; l1++) {
+                        if (notes[noteBufPtr][3][l1])
+                            removeNote(&notes[noteBufPtr][0][l1], -1, 0);
+                            releaseNoteCount--;
+                    }
+                }
 
-        for (int l3 = 0; l3 < 4; l3++) {
-            for (int l2 = noteCount; l2 > index; l2--) {
-                notes[bufPtr][l3][l2] = notes[bufPtr][l3][l2 - 1];
             }
         }
-    }
-    notes[bufPtr][0][index] = note;
-    notes[bufPtr][1][index] = velocity;
-    notes[bufPtr][2][index] = tick;
-    notes[bufPtr][3][index] = 0;
-    noteCount++;
+        // modify buffer that is not accessed by arpeggio output
+        bufPtr = (noteBufPtr) ? 0 : 1;
 
-    if (repeatPatternThroughChord == 2) noteOfs = noteCount - 1;
-    copyNoteBuffer();
-    mutex.unlock();
-}
-
-void MidiArp::handleNoteOff(int note, int tick, int keep_rel)
-{
-    int bufPtr, index;
-
-    // modify buffer that is not accessed by arpeggio output
-    bufPtr = (noteBufPtr) ? 0 : 1;
-    if (!noteCount) {
-        return;
-    }
-    if (sustain) {
-        sustainBuffer.append(note);
-        return;
-    }
-
-    if (latch_mode) {
-        latchBuffer.append(note);
-        if (latchBuffer.count() == noteCount) {
-            latchTimer->stop();
-        }
+        if (!noteCount || (inEv.data > notes[bufPtr][0][noteCount - 1]))
+            index = noteCount;
         else {
-            latchTimer->start(200);
-        }
-        return;
-    }
-
-    if ((!keep_rel) || (!release_time)) {
-        //definitely remove from buffer
-        if (note == notes[bufPtr][0][noteCount - 1]) {
-            //note is on top of buffer: only decrement noteCount
-            noteCount--;
-            if (repeatPatternThroughChord == 2) noteOfs = noteCount - 1;
-        }
-        else {
-            //note is not on top: take out the note and pull down all above
             index = 0;
-            while ((index < noteCount) && (note > notes[bufPtr][0][index])) index++;
-            deleteNoteAt(index, bufPtr);
+            while (inEv.data > notes[bufPtr][0][index]) index++;
+
+            for (int l3 = 0; l3 < 4; l3++) {
+                for (int l2 = noteCount; l2 > index; l2--) {
+                    notes[bufPtr][l3][l2] = notes[bufPtr][l3][l2 - 1];
+                }
+            }
+        }
+        notes[bufPtr][0][index] = inEv.data;
+        notes[bufPtr][1][index] = inEv.value;
+        notes[bufPtr][2][index] = tick;
+        notes[bufPtr][3][index] = 0;
+        noteCount++;
+
+        if (repeatPatternThroughChord == 2) noteOfs = noteCount - 1;
+        if ((trigByKbd && (noteCount == 1))) {
+            nextTick = tick + 2; //schedDelayTicks;
+            gotKbdTrig = true;
         }
     }
-    else tagAsReleased(note, tick, bufPtr);
+    else {
+        // This is a NOTE OFF event
+        // modify buffer that is not accessed by arpeggio output
+        bufPtr = (noteBufPtr) ? 0 : 1;
+        if (!noteCount) {
+            return(false);
+        }
+        if (sustain) {
+            sustainBuffer.append(inEv.data);
+            return(false);
+        }
 
+        if (latch_mode) {
+            latchBuffer.append(inEv.data);
+            if (latchBuffer.count() == noteCount) {
+                latchTimer->stop();
+            }
+            else {
+                latchTimer->start(200);
+            }
+            return(false);
+        }
+
+        if ((!keep_rel) || (!release_time)) {
+            //definitely remove from buffer
+            if (inEv.data == notes[bufPtr][0][noteCount - 1]) {
+                //note is on top of buffer: only decrement noteCount
+                noteCount--;
+                if (repeatPatternThroughChord == 2) noteOfs = noteCount - 1;
+            }
+            else {
+                //note is not on top: take out the note and pull down all above
+                index = 0;
+                while ((index < noteCount) && (inEv.data > notes[bufPtr][0][index])) index++;
+                deleteNoteAt(index, bufPtr);
+            }
+        }
+        else tagAsReleased(inEv.data, tick, bufPtr);
+    }
     copyNoteBuffer();
+    return(false);
 }
 
 void MidiArp::removeNote(int *noteptr, int tick, int keep_rel)
@@ -244,6 +310,7 @@ void MidiArp::tagAsReleased(int note, int tick, int bufPtr)
         notes[bufPtr][3][l1] = 1;
         notes[bufPtr][2][l1] = tick;
     }
+    releaseNoteCount++;
 }
 
 void MidiArp::copyNoteBuffer()
@@ -272,7 +339,7 @@ void MidiArp::getNote(int *tick, int note[], int velocity[], int *length)
     tmpIndex[1] = -1;
     gotCC = false;
     pause = false;
-
+    if (restartFlag) advancePatternIndex(true);
 
     if (!patternIndex) initLoop();
     do {
@@ -349,8 +416,7 @@ void MidiArp::getNote(int *tick, int note[], int velocity[], int *length)
         if ((release_time > 0) && (notes[noteBufPtr][3][noteIndex[l1]])) {
             releasefn = 1.0 - (double)(arpTick
                     - notes[noteBufPtr][2][noteIndex[l1]])
-                    / release_time / (double)TPQN
-                    * 60 / queueTempo;
+                    / (release_time * (double)TPQN * 2);
 
             if (releasefn < 0.0) releasefn = 0.0;
         }
@@ -360,8 +426,7 @@ void MidiArp::getNote(int *tick, int note[], int velocity[], int *length)
             if (!notes[noteBufPtr][3][noteIndex[l1]]) {
                 attackfn = (double)(arpTick
                     - notes[noteBufPtr][2][noteIndex[l1]])
-                    / attack_time / (double)TPQN
-                    * 60 / queueTempo;
+                    / (attack_time * (double)TPQN * 2);
 
                 if (attackfn > 1.0) attackfn = 1.0;
                 old_attackfn[noteIndex[l1]] = attackfn;
@@ -374,13 +439,13 @@ void MidiArp::getNote(int *tick, int note[], int velocity[], int *length)
                 * vel * (1.0 + 0.005 * (double)(randomVelocity + grooveTmp))
                 * releasefn * attackfn, 0, 127, &outOfRange);
 
-        if ((notes[noteBufPtr][3][noteIndex[l1]]) && (!velocity[l1]))
-
+        if ((notes[noteBufPtr][3][noteIndex[l1]]) && (!velocity[l1])) {
             removeNote(&notes[noteBufPtr][0][noteIndex[l1]], -1, 0);
-
-        else
-
+            releaseNoteCount--;
+        }
+        else {
             l1++;
+        }
 
     } while ((tmpIndex[l1] >= 0) && (l1 < MAXCHORD - 1) && (l1 < noteCount));
 
@@ -389,9 +454,16 @@ void MidiArp::getNote(int *tick, int note[], int velocity[], int *length)
     *length = clip(len * stepWidth * (double)TPQN
             * (1.0 + 0.005 * (double)(randomLength + grooveTmp)), 2,
             1000000,  &outOfRange);
-    grooveTmp = (grooveIndex % 2) ? -grooveTick : grooveTick;
-    arpTick += stepWidth * (double)TPQN
-        * (1.0 + 0.005 * (double)grooveTmp);
+
+    if (!grooveIndex) grooveTick = newGrooveTick;
+    grooveTmp = TPQN * stepWidth * grooveTick * 0.01;
+    /** pairwise application of new groove shift */
+    if (grooveIndex % 2) {
+        grooveTmp = -grooveTmp;
+        grooveTick = newGrooveTick;
+    }
+    arpTick += stepWidth * TPQN + grooveTmp;
+
     *tick = arpTick + clip(stepWidth * 0.25 * (double)randomTick, 0,
             1000, &outOfRange);
 
@@ -409,6 +481,7 @@ bool MidiArp::advancePatternIndex(bool reset)
     }
     if ((patternIndex >= patternLen) || reset) {
         patternIndex = 0;
+        restartFlag = false;
         switch (repeatPatternThroughChord) {
             case 1:
                 noteOfs++;
@@ -440,46 +513,12 @@ void MidiArp::initLoop()
     grooveIndex = 0;
 }
 
-void MidiArp::prepareNextNote(int askedTick)
-{
-    int l1 = 0;
-    returnNote.clear();
-    returnVelocity.clear();
-
-    nextNoteTick = askedTick;
-    while ((nextNote[l1] >= 0) && (l1 < MAXCHORD - 1)) {
-        returnNote.append(nextNote[l1]);
-        returnVelocity.append(nextVelocity[l1]);
-        l1++;
-    }
-    returnNote.append(-1); // mark end of chord
-    returnLength = nextLength;
-    returnIsNew = newNext;
-    newNext = false;
-}
-
-int MidiArp::getNextNoteTick()
-{
-    return(nextNoteTick);
-}
-
-bool MidiArp::wantTrigByKbd()
-{
-    bool on = ((getPressedNoteCount() == 1) && trigByKbd);
-    return(on);
-}
-
 void MidiArp::prepareCurrentNote(int askedTick)
 {
+    newRandomValues();
+    gotKbdTrig = false;
     currentTick = askedTick;
-    start(Priority(6));
-    wait();
-}
-
-void MidiArp::run()
-{
     int l1 = 0;
-    mutex.lock();
     updateNotes();
     returnTick = currentNoteTick;
     returnNote.clear();
@@ -494,7 +533,6 @@ void MidiArp::run()
     returnLength = currentLength;
     returnIsNew = newCurrent;
     newCurrent = false;
-    mutex.unlock();
 }
 
 void MidiArp::updateNotes()
@@ -503,8 +541,8 @@ void MidiArp::updateNotes()
 
     //allow 8 ticks of tolerance for echo tick for external sync
     if ((currentTick + 8) >= currentNoteTick) {
-        currentNoteTick = nextNoteTick;
-        getNote(&nextNoteTick, nextNote, nextVelocity, &nextLength);
+        currentNoteTick = nextTick;
+        getNote(&nextTick, nextNote, nextVelocity, &nextLength);
         while ((nextNote[l1] >= 0) && (l1 < MAXCHORD - 1)) {
             currentNote[l1] = nextNote[l1];
             currentVelocity[l1] = nextVelocity[l1];
@@ -521,7 +559,6 @@ void MidiArp::foldReleaseTicks(int tick)
 {
     int bufPtr, l2;
 
-    mutex.lock();
     bufPtr = (noteBufPtr) ? 0 : 1;
 
     for (l2 = 0; l2 < noteCount; l2++) {
@@ -529,7 +566,6 @@ void MidiArp::foldReleaseTicks(int tick)
     }
 
     copyNoteBuffer();
-    mutex.unlock();
 }
 
 void MidiArp::initArpTick(int tick)
@@ -537,7 +573,7 @@ void MidiArp::initArpTick(int tick)
     arpTick = tick;
     currentVelocity[0] = 0;
     currentNoteTick = tick;
-    nextNoteTick  = tick;
+    nextTick  = tick;
     nextVelocity[0] = 0;
     noteIndex[0] = -1;
     patternIndex = 0;
@@ -546,7 +582,6 @@ void MidiArp::initArpTick(int tick)
 
 void MidiArp::updatePattern(const QString& p_pattern)
 {
-    mutex.lock();
     int l1;
     QChar c;
 
@@ -574,7 +609,6 @@ void MidiArp::updatePattern(const QString& p_pattern)
     patternIndex = 0;
     grooveIndex = 0;
     noteOfs = 0;
-    mutex.unlock();
 }
 
 int MidiArp::clip(int value, int min, int max, bool *outOfRange)
@@ -594,14 +628,12 @@ int MidiArp::clip(int value, int min, int max, bool *outOfRange)
 
 void MidiArp::newRandomValues()
 {
-    mutex.lock();
     randomTick = (double)randomTickAmp * (0.5 - (double)random()
             / (double)RAND_MAX);
     randomVelocity = (double)randomVelocityAmp * (0.5 - (double)random()
             / (double)RAND_MAX);
     randomLength = (double)randomLengthAmp * (0.5 - (double)random()
             / (double)RAND_MAX);
-    mutex.unlock();
 }
 
 void MidiArp::updateRandomTickAmp(int val)
@@ -629,41 +661,22 @@ void MidiArp::updateReleaseTime(int val)
     release_time = (double)val;
 }
 
-void MidiArp::updateQueueTempo(int val)
-{
-    queueTempo = (double)val;
-}
-
 void MidiArp::updateTriggerMode(int val)
 {
-    switch (val) {
-        case 0:
-            trigByKbd = false;
-            restartByKbd = false;
-        break;
-        case 1:
-            trigByKbd = false;
-            restartByKbd = true;
-        break;
-        case 2:
-            trigByKbd = true;
-            restartByKbd = true;
-        break;
-        default:
-            trigByKbd = false;
-            restartByKbd = false;
-    }
+    trigByKbd = val&2;
+    restartByKbd = val&1 || val&2;
 }
 
 void MidiArp::clearNoteBuffer()
 {
     noteCount = 0;
     latchBuffer.clear();
+    releaseNoteCount = 0;
 }
 
 int MidiArp::getPressedNoteCount()
 {
-    int c = noteCount - latchBuffer.count();
+    int c = noteCount - latchBuffer.count() - releaseNoteCount;
     return(c);
 }
 
@@ -703,9 +716,7 @@ void MidiArp::purgeLatchBuffer()
 void MidiArp::newGrooveValues(int p_grooveTick, int p_grooveVelocity,
         int p_grooveLength)
 {
-    mutex.lock();
-    grooveTick = p_grooveTick;
+    newGrooveTick = p_grooveTick;
     grooveVelocity = p_grooveVelocity;
     grooveLength = p_grooveLength;
-    mutex.unlock();
 }
