@@ -29,7 +29,6 @@
 #include <QObject>
 #include <QString>
 #include <QVector>
-#include <alsa/asoundlib.h>
 #include <main.h>
 
 #ifndef SAMPLE_H
@@ -49,9 +48,9 @@
  * for controller data as a QObject.
  *
  * The parameters of MidiLfo are controlled by the LfoWidget class.
- * A pointer to MidiLfo is passed to the SeqDriver thread, which calls
+ * A pointer to MidiLfo is passed to the Engine, which calls
  * the MidiLfo::getNextFrame member as a function of the position of
- * the ALSA queue. MidiLfo will return an array of controller values
+ * the Driver's queue. MidiLfo will return an array of controller values
  * representing a frame of its internal MidiLfo::data buffer. This frame
  * has size 1 except for resolution higher than 16th notes.
  * The MidiLfo::data buffer is populated by the MidiLfo::getData function
@@ -66,12 +65,17 @@ class MidiLfo : public QObject  {
   Q_OBJECT
 
   private:
-    double queueTempo;  /*!< current tempo of the ALSA queue, not in use here */
+    double queueTempo;  /*!< current tempo of the transport, not in use here */
     int lastMouseLoc;   /*!< The X location of the last modification of the wave, used for interpolation*/
     int lastMouseY;     /*!< The Y location at the last modification of the wave, used for interpolation*/
     int frameptr;       /*!< position of the currently output frame in the MidiArp::data waveform */
+    bool reverse;       /*!< True when the play direction is backwards */
+    bool pingpong;       /*!< True when the play direction alternates */
+    int curLoopMode;    /*!< Local storage of the currently active Loop mode */
     int recValue;
     int lastSampleValue;
+    bool seqFinished;   /*!< When True, all output events are muted, used when NOTE OFF is received */
+    int noteCount;      /*!< The number of keys currently pressed on keyboard */
 /**
  * @brief This function allows forcing an integer value within the
  * specified range (clip).
@@ -93,8 +97,15 @@ class MidiLfo : public QObject  {
     void updateCustomWaveOffset(int cwoffs);
 
   public:
-    int portOut;    /*!< ALSA output port number */
-    int channelOut; /*!< ALSA output channel */
+    bool enableNoteOff;
+    bool enableVelIn;
+    bool restartByKbd;
+    bool trigByKbd;
+    bool enableLoop;
+    bool gotKbdTrig;
+    bool restartFlag; /*!< Signals frameptr reset on next getNextFrame() call */
+    int portOut;    /*!< MIDI output port number */
+    int channelOut; /*!< MIDI output channel */
     bool recordMode, isRecording;
     int old_res;
     int ccnumber;   /*!< MIDI Controller CC number to output */
@@ -110,6 +121,8 @@ class MidiLfo : public QObject  {
                                     @par 4: Square
                                     @par 5: Use Custom Wave */
     int cwmin;                  /*!< The minimum of MidiLfo::customWave */
+    int nextTick;
+    int newGrooveTick, grooveTick, grooveVelocity, grooveLength, grooveIndex;
     QVector<Sample> customWave; /*!< Vector of Sample points holding the custom drawn wave */
     QVector<bool> muteMask;     /*!< Vector of booleans with mute state information for each wave point */
 
@@ -122,12 +135,13 @@ class MidiLfo : public QObject  {
     void updateOffset(int);
     void updateResolution(int);
     void updateSize(int);
+    void updateLoop(int);
     void updateQueueTempo(int);
     void record(int value);
 /*! @brief This function sets MidiLfo::isMuted, which is checked by
- * SeqDriver and which suppresses data output globally if set to True.
+ * Engine and which suppresses data output globally if set to True.
  *
- * @param on Set to True to suppress data output to ALSA
+ * @param on Set to True to suppress data output to the Driver
  */
     void setMuted(bool on);
 /*! @brief This function sets the (controller) value of one point of the
@@ -180,22 +194,22 @@ class MidiLfo : public QObject  {
  * while in calculated waveform mode. (MidiLfo::waveFormIndex 1 ... 4).
  */
     void copyToCustom();
-/*! @brief This function resets the MidiLfo::frameptr to zero.
+/*! @brief This function sets the MidiLfo::frameptr to the given value.
  *
- * It is called when the ALSA queue starts.
+ * It is called when the Transport starts.
+ * @param idx Index to which the frameptr is set
  */
-    void resetFramePtr();
+    void setFramePtr(int idx);
 /**
- * @brief This function checks whether an ALSA event is eligible for this
- * module.
+ * @brief This function does the actions related to a newly received event.
  *
- * Its response depends on the input filter settings, i.e. note,
- * velocity and channel.
- *
- * @param inEv MidiEvent to check
- * @return True if inEv is in the input range of the module
+ * It is called by Engine when a new event is received on the MIDI input port.
+
+ * @param inEv MidiEvent to check and process or not
+ * @param tick The time the event was received in internal ticks
+ * @return True if inEv is in not the input range of the module (event is unmatched)
  */
-    bool wantEvent(MidiEvent inEv);
+    bool handleEvent(MidiEvent inEv, int tick);
 /*! @brief This function is the main calculator for the data contained
  * in a waveform.
  *
@@ -212,7 +226,7 @@ class MidiLfo : public QObject  {
  *
  * @param *p_data reference to an array the frame is copied to
  */
-    void getNextFrame(QVector<Sample> *p_data);
+    void getNextFrame(QVector<Sample> *p_data, int tick);
 /*! @brief This function toggles the mute state of one point of the
  * MidiLfo::muteMask array.
  *
@@ -227,6 +241,17 @@ class MidiLfo : public QObject  {
  * @see MidiLfo::setMutePoint
  */
     bool toggleMutePoint(double mouseX);
+/**
+ * @brief This function copies the new values transferred from the
+ * GrooveWidget into variables used by MidiArp::getNote.
+ *
+ * @param p_grooveTick Groove amount for timing displacements
+ * @param p_grooveVelocity Groove amount for velocity variations
+ * @param p_grooveLength Groove amount for note length variations
+ */
+    void newGrooveValues(int p_grooveTick, int p_grooveVelocity,
+            int p_grooveLength);
+    int getFramePtr() { return frameptr;}
 
   signals:
     void nextStep(int frameptr);
