@@ -34,9 +34,14 @@ MidiSeq::MidiSeq()
     recordMode = false;
     trigByKbd = false;
     restartByKbd = false;
-    enableLoop = false;
+    enableLoop = true;
+    gotKbdTrig = false;
     currentRecStep = 0;
     seqFinished = false;
+    reverse = false;
+    pingpong = false;
+    restartFlag = false;
+    curLoopMode = 0;
     noteCount = 0;
 
     nOctaves = 4;
@@ -53,6 +58,11 @@ MidiSeq::MidiSeq()
     channelOut = 0;
     waveFormIndex = 0;
     currentIndex = 0;
+    nextTick = 0;
+    grooveTick = 0;
+    newGrooveTick = 0;
+    grooveVelocity = 0;
+    grooveLength = 0;
     isMuted = false;
     int lt = 0;
     int l1 = 0;
@@ -77,65 +87,132 @@ void MidiSeq::setMuted(bool on)
     isMuted = on;
 }
 
-bool MidiSeq::wantEvent(MidiEvent inEv) {
-
-    if ((inEv.type != EV_NOTEON) && (inEv.type != EV_CONTROLLER)) return(false);
-
-    if (inEv.channel != chIn) return(false);
-
-    if ((inEv.type == EV_NOTEON)) {
-        if ((inEv.data < 36) || (inEv.data >= 84)) return(false);
-    }
-    return(true);
-}
-
-void MidiSeq::handleNote(int note, int velocity, int tick)
+bool MidiSeq::handleEvent(MidiEvent inEv, int tick)
 {
-    if (recordMode) recordNote(note);
+    if (inEv.type != EV_NOTEON) return(true);
+    if (inEv.channel != chIn) return(true);
+    if ((inEv.data < 36) || (inEv.data >= 84)) return(true);
 
+    if (inEv.value) {
+        /*This is a NOTE ON event*/
+        if (recordMode) {
+            recordNote(inEv.data);
+            emit noteEvent(inEv.data, inEv.value);
+            return(false);
+        }
+        if (enableNoteIn) updateTranspose(inEv.data - 60);
+        if (restartByKbd && !noteCount) restartFlag = true;
+        if (enableVelIn) updateVelocity(inEv.value);
+        seqFinished = false;
+        noteCount++;
+        if ((trigByKbd && (noteCount == 1))) {
+            nextTick = tick + 2; //schedDelayTicks;
+            gotKbdTrig = true;
+        }
+        emit noteEvent(inEv.data, inEv.value);
+    }
     else {
-        if (velocity) {
-            /**This is a NOTE ON event*/
-            if (enableNoteIn) updateTranspose(note - 60);
-            if (restartByKbd && !noteCount) currentIndex = 0;
-            if (enableVelIn) updateVelocity(velocity);
-            seqFinished = false;
-            noteCount++;
-        }
-        else {
-            /**This is a NOTE OFF event*/
-            if (enableNoteOff && (noteCount == 1)) seqFinished = true;
-            if (noteCount) noteCount--;
-        }
+        /*This is a NOTE OFF event*/
+        if (enableNoteOff && (noteCount == 1)) seqFinished = true;
+        if (noteCount) noteCount--;
     }
 
-    if (velocity) emit noteEvent(note, velocity);
+    return(false);
 }
 
-bool MidiSeq::wantTrigByKbd()
-{
-    bool on = (trigByKbd && (noteCount == 1));
-    return(on);
-}
-
-
-void MidiSeq::getNextNote(Sample *p_sample)
+void MidiSeq::getNextNote(Sample *p_sample, int tick)
 {
+    const int frame_nticks = TPQN / res;
     Sample sample;
+    int cur_grv_sft;
+
+    gotKbdTrig = false;
+    if (restartFlag) setCurrentIndex(0);
+    if (!currentIndex) grooveTick = newGrooveTick;
+    if (!seqFinished) emit nextStep(currentIndex);
+
     sample = customWave.at(currentIndex);
-    emit nextStep(currentIndex);
-    currentIndex++;
-    currentIndex %= (size * res);
-    if (!enableLoop && !currentIndex) seqFinished = true;
+    advancePatternIndex();
+
+    if (nextTick < (tick - frame_nticks)) nextTick = tick;
+
+    sample.value+=transp;
+    sample.tick = nextTick;
     if (seqFinished) sample.muted = true;
+
+
+    cur_grv_sft = 0.01 * (grooveTick * frame_nticks);
+
+    /* pairwise application of new groove shift */
+    if (!(currentIndex % 2)) {
+        cur_grv_sft = -cur_grv_sft;
+        grooveTick = newGrooveTick;
+    }
+    nextTick += frame_nticks + cur_grv_sft;
+
+    if (!trigByKbd && !(currentIndex % 2)) {
+        /* round-up to current resolution (quantize) */
+        nextTick/=frame_nticks;
+        nextTick*=frame_nticks;
+    }
+
     *p_sample = sample;
 }
 
+void MidiSeq::advancePatternIndex()
+{
+    const int npoints = res * size;
+    int pivot = abs(loopMarker);
+
+    if (reverse) {
+        currentIndex--;
+        if (currentIndex == -1) {
+            if (pingpong) {
+                reverse = false;
+                currentIndex = 0;
+            }
+            else currentIndex = npoints - 1;
+        }
+        else if (currentIndex == pivot - 1) {
+            if (!enableLoop) seqFinished = true;
+            if (loopMarker < 0) pingpong = true;
+            if (loopMarker > 0) pingpong = false;
+            if (pingpong) {
+                reverse = false;
+                currentIndex = pivot;
+            }
+            else currentIndex = npoints - 1;
+        }
+    }
+    else {
+        currentIndex++;
+        if (!pivot) pivot = npoints;
+        if (currentIndex == npoints) {
+            if (pingpong) {
+                reverse = true;
+                currentIndex = npoints - 1;
+            }
+            else currentIndex = 0;
+        }
+        else if ((currentIndex == pivot)) {
+            if (!enableLoop) seqFinished = true;
+            if (loopMarker > 0) pingpong = true;
+            if (loopMarker < 0) pingpong = false;
+            if (pingpong) {
+                reverse = true;
+                currentIndex = pivot - 1;
+            }
+            else currentIndex = 0;
+        }
+    }
+}
+
 void MidiSeq::getData(QVector<Sample> *p_data)
 {
     Sample sample;
     int lt = 0;
-    int step = TPQN / res;
+    const int step = TPQN / res;
+    const int npoints = res * size;
 
     //res: number of events per beat
     //size: size of waveform in beats
@@ -144,8 +221,8 @@ void MidiSeq::getData(QVector<Sample> *p_data)
 
     switch(waveFormIndex) {
         case 0: //custom
-            lt = step * customWave.count();
-            data = customWave;
+            lt = step * npoints;
+            data = customWave.mid(0, npoints);
         break;
         default:
         break;
@@ -176,6 +253,14 @@ void MidiSeq::updateWaveForm(int val)
     waveFormIndex = val;
 }
 
+void MidiSeq::updateLoop(int val)
+{
+    reverse = val&1;
+    pingpong = val&2;
+    enableLoop = !(val&4);
+    curLoopMode = val;
+}
+
 void MidiSeq::updateVelocity(int val)
 {
     vel = val;
@@ -204,6 +289,14 @@ void MidiSeq::setCustomWavePoint(double mouseX, double mouseY)
     setRecordedNote(12 * (mouseY * nOctaves + baseOctave));
 }
 
+void MidiSeq::setLoopMarker(double mouseX)
+{
+    const int npoints = res * size;
+    if (mouseX > 0) loopMarker = mouseX * (double)npoints + .5;
+    else loopMarker = mouseX * (double)npoints - .5;
+    if (abs(loopMarker) >= npoints) loopMarker = 0;
+}
+
 void MidiSeq::setRecordMode(int on)
 {
     recordMode = on;
@@ -224,22 +317,26 @@ void MidiSeq::resizeAll()
     int lt = 0;
     int l1 = 0;
     int os;
-    int step = TPQN / res;
+    const int step = TPQN / res;
+    const int npoints = res * size;
     Sample sample;
 
-    currentIndex%=(res * size);
+    currentIndex%=npoints;
+    currentRecStep%=npoints;
+
     os = customWave.count();
-    customWave.resize(size * res);
-    muteMask.resize(size * res);
-    for (l1 = 0; l1 < customWave.count(); l1++) {
-        if (l1 >= os) muteMask.replace(l1, muteMask.at(l1 % os));
-        sample = customWave.at(l1 % os);
-        sample.tick = lt;
-        sample.muted = muteMask.at(l1);
-        customWave.replace(l1, sample);
-        lt+=step;
+    if (os < npoints) {
+        customWave.resize(npoints);
+        muteMask.resize(npoints);
+        for (l1 = 0; l1 < npoints; l1++) {
+            if (l1 >= os) muteMask.replace(l1, muteMask.at(l1 % os));
+            sample = customWave.at(l1 % os);
+            sample.tick = lt;
+            sample.muted = muteMask.at(l1);
+            customWave.replace(l1, sample);
+            lt+=step;
+        }
     }
-    currentRecStep %= (res * size);
 }
 
 void MidiSeq::copyToCustom()
@@ -282,8 +379,21 @@ void MidiSeq::setMutePoint(double mouseX, bool on)
 void MidiSeq::setCurrentIndex(int ix)
 {
     currentIndex=ix;
+
     if (!ix) {
-        seqFinished = false;
-        noteCount = 0;
+        reverse = curLoopMode&1;
+        seqFinished = (enableNoteOff && !noteCount);
+        restartFlag = false;
+        if (reverse) currentIndex = res * size - 1;
     }
 }
+
+void MidiSeq::newGrooveValues(int p_grooveTick, int p_grooveVelocity,
+        int p_grooveLength)
+{
+    // grooveTick is only updated on pair steps to keep quantization
+    // newGrooveTick stores the GUI value temporarily
+    newGrooveTick = p_grooveTick;
+    grooveVelocity = p_grooveVelocity;
+    grooveLength = p_grooveLength;
+}
