@@ -40,8 +40,12 @@ SeqScreen::SeqScreen(QWidget* parent) : QWidget (parent)
     baseOctave = 3;
     nOctaves= 4;
     recordMode = false;
-    currentRecStep = false;
+    currentRecStep = 0;
+    loopMarker = 0;
     currentIndex = 0;
+    grooveTick = 0;
+    grooveVelocity = 0;
+    grooveLength = 0;
     isMuted = false;
 }
 
@@ -170,8 +174,7 @@ void SeqScreen::paintEvent(QPaintEvent*)
     pen.setWidth(notestreak_thick);
     p.setPen(pen);
     for (l1 = 0; l1 < npoints; l1++) {
-
-        x = l1 * xscale * nsteps / npoints;
+        x = (l1 + .01 * (double)grooveTick * (l1 % 2)) * nsteps * xscale / npoints;
         tmpval = p_data.at(l1).value;
         if ((tmpval >= 12 * baseOctave) && (tmpval < 12 * maxOctave)) {
             ypos = yscale - yscale
@@ -180,12 +183,11 @@ void SeqScreen::paintEvent(QPaintEvent*)
             xpos = SEQSCR_HMARG + x + pen.width() / 2;
             if (p_data.at(l1).muted) {
                 pen.setColor(QColor(5, 40, 100));
-                p.setPen(pen);
             }
             else {
                 pen.setColor(QColor(50, 130, 180));
-                p.setPen(pen);
             }
+            p.setPen(pen);
             p.drawLine(xpos, ypos,
                             xpos + (xscale / beatRes) - pen.width(), ypos);
         }
@@ -194,18 +196,36 @@ void SeqScreen::paintEvent(QPaintEvent*)
     pen.setWidth(2);
     pen.setColor(QColor(50, 160, 220));
     p.setPen(pen);
-    p.drawLine(SEQSCR_HMARG / 2, ypos,
-                        SEQSCR_HMARG *2 / 3, ypos);
+    p.drawLine(SEQSCR_HMARG / 2, ypos, SEQSCR_HMARG *2 / 3, ypos);
 
     // Cursor
     pen.setWidth(notestreak_thick);
     pen.setColor(QColor(50, 180, 220));
     p.setPen(pen);
-    x = currentIndex * xscale * (int)nsteps / npoints;
+    x = (currentIndex + .01 * (double)grooveTick * (currentIndex % 2))
+        * xscale * (int)nsteps / npoints;
     xpos = SEQSCR_HMARG + x + pen.width() / 2;
     p.drawLine(xpos, h - 2,
                     xpos + (xscale / beatRes) - pen.width(), h - 2);
-
+    // Loop Marker
+    if (loopMarker) {
+        QPolygon trg;
+        pen.setWidth(2);
+        pen.setColor(QColor(80, 250, 120));
+        p.setPen(pen);
+        x = (abs(loopMarker) + .01 * (double)grooveTick
+            * (currentIndex % 2))* xscale * (int)nsteps / npoints;
+        xpos = SEQSCR_HMARG + x + pen.width() / 2;
+        ypos = h - SEQSCR_VMARG;
+        tmpval = SEQSCR_VMARG / 2;
+        trg << QPoint(xpos, ypos);
+        if (loopMarker > 0)
+            trg << QPoint(xpos - tmpval, ypos + tmpval);
+        else
+            trg << QPoint(xpos + tmpval, ypos + tmpval);
+        trg << QPoint(xpos, h);
+        p.drawPolygon(trg, Qt::WindingFill);
+    }
 }
 
 void SeqScreen::updateScreen(const QVector<Sample>& data)
@@ -220,6 +240,14 @@ void SeqScreen::updateScreen(int p_index)
     update();
 }
 
+void SeqScreen::newGrooveValues(int tick, int vel, int length)
+{
+    grooveTick = tick;
+    grooveVelocity = vel;
+    grooveLength = length;
+    update();
+}
+
 void SeqScreen::setMuted(bool on)
 {
     isMuted = on;
@@ -245,10 +273,6 @@ void SeqScreen::mousePressEvent(QMouseEvent *event)
 {
     mouseX = event->x();
     mouseY = event->y();
-    if ((mouseX < SEQSCR_HMARG)|| (mouseX >= w - SEQSCR_HMARG))
-        return;
-    if ((mouseY <= SEQSCR_VMARG)|| (mouseY > h - SEQSCR_VMARG))
-        return;
     emit mousePressed(((double)mouseX - SEQSCR_HMARG) /
                             (w - 2 * SEQSCR_HMARG),
                 1. - ((double)mouseY - SEQSCR_VMARG) /
@@ -265,6 +289,12 @@ void SeqScreen::setCurrentRecStep(int recStep)
     currentRecStep = recStep;
 }
 
+void SeqScreen::setLoopMarker(int pos)
+{
+    loopMarker = pos;
+    update();
+}
+
 QSize SeqScreen::sizeHint() const
 {
     return QSize(SEQSCR_MIN_W, SEQSCR_MIN_H);
