@@ -42,7 +42,7 @@ SeqWidget::SeqWidget(MidiSeq *p_midiWorker, int portCount, bool compactStyle,
     int l1;
     QStringList midiCCNames;
     midiCCNames << "MuteToggle" << "Velocity" << "NoteLength"
-                << "RecordToggle" << "Resolution"<< "Size" << "unknown";
+                << "RecordToggle" << "Resolution"<< "Size" << "LoopMode" << "unknown";
     midiControl = new MidiControl(midiCCNames);
 
     manageBox = new ManageBox("Seq:", true, this);
@@ -52,7 +52,8 @@ SeqWidget::SeqWidget(MidiSeq *p_midiWorker, int portCount, bool compactStyle,
 
     QSignalMapper *dispSignalMapper = new QSignalMapper(this);
     QLabel *dispLabel[4];
-    QString dispText[4] = {tr("&Full"), tr("&Upper"), tr("&Mid"), tr("&Lower")};
+    QString dispText[4] = {tr("&F"), tr("&U"), tr("&M"), tr("&L")};
+    QString dispToolTip[4] = {tr("Full"), tr("Upper"), tr("Mid"), tr("Lower")};
     QGridLayout *dispBoxLayout = new QGridLayout;
 
     for (int l1 = 0; l1 < 4; l1++) {
@@ -62,8 +63,9 @@ SeqWidget::SeqWidget(MidiSeq *p_midiWorker, int portCount, bool compactStyle,
         dispSignalMapper->setMapping(dispVert[l1], l1);
         dispVert[l1]->setAutoExclusive(true);
         dispLabel[l1]->setBuddy(dispVert[l1]);
-        dispBoxLayout->addWidget(dispLabel[l1], l1, 0);
-        dispBoxLayout->addWidget(dispVert[l1], l1, 1);
+        dispVert[l1]->setToolTip(dispToolTip[l1]);
+        dispBoxLayout->addWidget(dispLabel[l1], 0, l1);
+        dispBoxLayout->addWidget(dispVert[l1], 1, l1);
     }
 
     dispVert[0]->setChecked(true);
@@ -114,18 +116,11 @@ SeqWidget::SeqWidget(MidiSeq *p_midiWorker, int portCount, bool compactStyle,
     enableTrigByKbdLabel->setBuddy(enableTrigByKbd);
     enableTrigByKbd->setToolTip(tr("Retrigger sequence when a new note is received"));
 
-    QLabel *enableLoopLabel = new QLabel(tr("&Loop"),inBox);
-    enableLoop = new QCheckBox(this);
-    connect(enableLoop, SIGNAL(toggled(bool)), this, SLOT(updateEnableLoop(bool)));
-    enableLoopLabel->setBuddy(enableLoop);
-    enableLoop->setToolTip(tr("Play sequence as loop instead of a single run"));
-
     enableNoteIn->setChecked(true);
     enableNoteOff->setChecked(false);
     enableVelIn->setChecked(true);
     enableRestartByKbd->setChecked(false);
     enableTrigByKbd->setChecked(false);
-    enableLoop->setChecked(true);
 
     QLabel *chInLabel = new QLabel(tr("&Channel"), inBox);
     chIn = new QComboBox(inBox);
@@ -145,10 +140,8 @@ SeqWidget::SeqWidget(MidiSeq *p_midiWorker, int portCount, bool compactStyle,
     inBoxLayout->addWidget(enableRestartByKbd, 3, 1);
     inBoxLayout->addWidget(enableTrigByKbdLabel, 4, 0);
     inBoxLayout->addWidget(enableTrigByKbd, 4, 1);
-    inBoxLayout->addWidget(enableLoopLabel, 5, 0);
-    inBoxLayout->addWidget(enableLoop, 5, 1);
-    inBoxLayout->addWidget(chInLabel, 6, 0);
-    inBoxLayout->addWidget(chIn, 6, 1);
+    inBoxLayout->addWidget(chInLabel, 5, 0);
+    inBoxLayout->addWidget(chIn, 5, 1);
     if (compactStyle) {
         inBoxLayout->setSpacing(1);
         inBoxLayout->setMargin(2);
@@ -227,6 +220,18 @@ SeqWidget::SeqWidget(MidiSeq *p_midiWorker, int portCount, bool compactStyle,
     connect(waveFormBox, SIGNAL(activated(int)), this,
             SLOT(updateWaveForm(int)));
 
+    loopBox = new QComboBox(seqBox);
+    QStringList names;
+    names.clear();
+    names << "->_>" << " <_<-" << "->_<" << " >_<-" << "->_|" << " |_<-";
+    loopBox->insertItems(0, names);
+    loopBox->setCurrentIndex(0);
+    loopBox->setToolTip(tr("Loop, bounce or play once going forward or backward"));
+    loopBox->setMinimumContentsLength(5);
+    connect(loopBox, SIGNAL(activated(int)), this,
+            SLOT(updateLoop(int)));
+    midiControl->addMidiLearnMenu(loopBox, 6);
+
     QLabel *recordButtonLabel = new QLabel(tr("Re&cord"), seqBox);
     recordAction = new QAction(QIcon(seqrecord_xpm), tr("Re&cord"), seqBox);
     recordAction->setToolTip(tr("Record step by step"));
@@ -241,7 +246,6 @@ SeqWidget::SeqWidget(MidiSeq *p_midiWorker, int portCount, bool compactStyle,
             seqBox);
     resBox = new QComboBox(seqBox);
     resBoxLabel->setBuddy(resBox);
-    QStringList names;
     names.clear();
     names << "1" << "2" << "4" << "8" << "16";
     resBox->insertItems(0, names);
@@ -302,15 +306,16 @@ SeqWidget::SeqWidget(MidiSeq *p_midiWorker, int portCount, bool compactStyle,
     }
 
     QGridLayout *paramBoxLayout = new QGridLayout;
-    paramBoxLayout->addWidget(waveFormBoxLabel, 0, 0);
-    paramBoxLayout->addWidget(waveFormBox, 0, 1);
-    paramBoxLayout->addWidget(recordButtonLabel, 1, 0);
-    paramBoxLayout->addWidget(recordButton, 1, 1);
-    paramBoxLayout->addWidget(resBoxLabel, 2, 0);
-    paramBoxLayout->addWidget(resBox, 2, 1);
-    paramBoxLayout->addWidget(sizeBoxLabel, 3, 0);
-    paramBoxLayout->addWidget(sizeBox, 3, 1);
-    paramBoxLayout->setRowStretch(4, 1);
+    paramBoxLayout->addWidget(loopBox, 0, 0, 1, 2);
+    paramBoxLayout->addWidget(waveFormBoxLabel, 1, 0);
+    paramBoxLayout->addWidget(waveFormBox, 1, 1);
+    paramBoxLayout->addWidget(recordButtonLabel, 2, 0);
+    paramBoxLayout->addWidget(recordButton, 2, 1);
+    paramBoxLayout->addWidget(resBoxLabel, 3, 0);
+    paramBoxLayout->addWidget(resBox, 3, 1);
+    paramBoxLayout->addWidget(sizeBoxLabel, 4, 0);
+    paramBoxLayout->addWidget(sizeBox, 4, 1);
+    paramBoxLayout->setRowStretch(5, 1);
 
     QGridLayout* seqBoxLayout = new QGridLayout;
     seqBoxLayout->addWidget(screen, 0, 0, 1, 2);
@@ -368,8 +373,6 @@ void SeqWidget::writeData(QXmlStreamWriter& xml)
                 midiWorker->restartByKbd));
             xml.writeTextElement("trigByKbd", QString::number(
                 midiWorker->trigByKbd));
-            xml.writeTextElement("enableLoop", QString::number(
-                midiWorker->enableLoop));
             xml.writeTextElement("channel", QString::number(
                 midiWorker->chIn));
         xml.writeEndElement();
@@ -384,6 +387,8 @@ void SeqWidget::writeData(QXmlStreamWriter& xml)
         xml.writeEndElement();
 
         xml.writeStartElement("seqParams");
+            xml.writeTextElement("loopmode", QString::number(
+                loopBox->currentIndex()));
             xml.writeTextElement("resolution", QString::number(
                 resBox->currentIndex()));
             xml.writeTextElement("size", QString::number(
@@ -414,6 +419,8 @@ void SeqWidget::writeData(QXmlStreamWriter& xml)
         }
         xml.writeStartElement("sequence");
             xml.writeTextElement("data", tempArray.toHex());
+            xml.writeTextElement("loopmarker", QString::number(
+                getLoopMarker()));
         xml.writeEndElement();
 
         midiControl->writeData(xml);
@@ -457,8 +464,6 @@ void SeqWidget::readData(QXmlStreamReader& xml)
                     enableRestartByKbd->setChecked(xml.readElementText().toInt());
                 else if (xml.name() == "trigByKbd")
                     enableTrigByKbd->setChecked(xml.readElementText().toInt());
-                else if (xml.name() == "enableLoop")
-                    enableLoop->setChecked(xml.readElementText().toInt());
                 else if (xml.name() == "channel") {
                     tmp = xml.readElementText().toInt();
                     chIn->setCurrentIndex(tmp);
@@ -494,6 +499,11 @@ void SeqWidget::readData(QXmlStreamReader& xml)
                 xml.readNext();
                 if (xml.isEndElement())
                     break;
+                if (xml.name() == "loopmode") {
+                    tmp = xml.readElementText().toInt();
+                    loopBox->setCurrentIndex(tmp);
+                    updateLoop(tmp);
+                }
                 else if (xml.name() == "resolution") {
                     tmp = xml.readElementText().toInt();
                     resBox->setCurrentIndex(tmp);
@@ -555,6 +565,11 @@ void SeqWidget::readData(QXmlStreamReader& xml)
                         lt+=step;
                     }
                 }
+                else if (xml.name() == "loopmarker") {
+                    tmp = xml.readElementText().toInt();
+                    midiWorker->loopMarker = tmp;
+                    screen->setLoopMarker(tmp);
+                }
                 else skipXmlElement(xml);
             }
         }
@@ -585,95 +600,6 @@ void SeqWidget::skipXmlElement(QXmlStreamReader& xml)
     }
 }
 
-void SeqWidget::readDataText(QTextStream& arpText)
-{
-    QString qs, qs2;
-    int l1, lt, wvtmp;
-    Sample sample;
-
-    qs = arpText.readLine();
-    qs2 = qs.section(' ', 0, 0);
-    enableNoteIn->setChecked(qs2.toInt());
-    qs2 = qs.section(' ', 1, 1);
-    enableVelIn->setChecked(qs2.toInt());
-    qs2 = qs.section(' ', 2, 2);
-    chIn->setCurrentIndex(qs2.toInt());
-    qs = arpText.readLine();
-    qs2 = qs.section(' ', 0, 0);
-    channelOut->setCurrentIndex(qs2.toInt());
-    qs2 = qs.section(' ', 1, 1);
-    portOut->setCurrentIndex(qs2.toInt());
-    qs2 = qs.section(' ', 2, 2);
-    notelength->setValue(qs2.toInt());
-    qs = arpText.readLine();
-    qs2 = qs.section(' ', 0, 0);
-    resBox->setCurrentIndex(qs2.toInt());
-    updateRes(qs2.toInt());
-    qs2 = qs.section(' ', 1, 1);
-    sizeBox->setCurrentIndex(qs2.toInt());
-    updateSize(qs2.toInt());
-    qs2 = qs.section(' ', 2, 2);
-    velocity->setValue(qs2.toInt());
-    qs2 = qs.section(' ', 3, 3);
-    transpose->setValue(qs2.toInt());
-    qs = arpText.readLine();
-    if (qs == "MIDICC")
-    {
-        qs = arpText.readLine();
-        while (qs != "EOCC") {
-            qs2 = qs.section(' ', 0, 0);
-            int controlID = qs2.toInt();
-            qs2 = qs.section(' ', 1, 1);
-            int ccnumber = qs2.toInt();
-            qs2 = qs.section(' ', 2, 2);
-            int channel = qs2.toInt();
-            qs2 = qs.section(' ', 3, 3);
-            int min = qs2.toInt();
-            qs2 = qs.section(' ', 4, 4);
-            int max = qs2.toInt();
-            midiControl->appendMidiCC(controlID, ccnumber, channel, min, max);
-            qs = arpText.readLine();
-        }
-    qs = arpText.readLine();
-    }
-
-    wvtmp = qs.toInt();
-
-    // Read Mute Mask
-    int step = TPQN / midiWorker->res;
-    qs = arpText.readLine();
-    if (qs.isEmpty() || (qs == "EOP")) return;
-    qs2 = qs.section(' ', 0, 0);
-    midiWorker->muteMask.clear();
-    l1 = 0;
-    while (qs2 !="EOM") {
-        midiWorker->muteMask.append(qs2.toInt());
-        l1++;
-        if (!(l1%32)) qs = arpText.readLine();
-        qs2 = qs.section(' ', l1%32, l1%32);
-    }
-
-    // Read Custom Waveform
-    qs = arpText.readLine();
-    qs2 = qs.section(' ', 0, 0);
-    midiWorker->customWave.clear();
-    l1 = 0;
-    lt = 0;
-    while (qs2 !="EOS") {
-        sample.value=qs2.toInt();
-        sample.tick = lt;
-        sample.muted = midiWorker->muteMask.at(l1);
-        midiWorker->customWave.append(sample);
-        lt+=step;
-        l1++;
-        if (!(l1%16)) qs = arpText.readLine();
-        qs2 = qs.section(' ', l1%16, l1%16);
-    }
-    waveFormBox->setCurrentIndex(wvtmp);
-    updateWaveForm(wvtmp);
-    modified = false;
-}
-
 void SeqWidget::loadWaveForms()
 {
     waveForms << tr("Custom");
@@ -733,12 +659,6 @@ void SeqWidget::updateEnableTrigByKbd(bool on)
     modified = true;
 }
 
-void SeqWidget::updateEnableLoop(bool on)
-{
-    midiWorker->enableLoop = on;
-    modified = true;
-}
-
 void SeqWidget::updateNoteLength(int val)
 {
     midiWorker->notelength = val + val;
@@ -784,6 +704,14 @@ void SeqWidget::updateSize(int val)
     modified = true;
 }
 
+void SeqWidget::updateLoop(int val)
+{
+    if (val > 5) return;
+    midiWorker->updateLoop(val);
+    modified = true;
+}
+
+
 void SeqWidget::updateVelocity(int val)
 {
     midiWorker->vel = val;
@@ -833,6 +761,14 @@ void SeqWidget::mouseMoved(double mouseX, double mouseY, int buttons)
 
 void SeqWidget::mousePressed(double mouseX, double mouseY, int buttons)
 {
+    if (mouseY < 0) {
+        if (mouseX < 0) mouseX = 0;
+        if (buttons == 2) mouseX = - mouseX;
+        midiWorker->setLoopMarker(mouseX);
+        screen->setLoopMarker(midiWorker->loopMarker);
+        modified = true;
+        return;
+    }
     if (buttons == 2) {
         lastMute = midiWorker->toggleMutePoint(mouseX);
     } else {
@@ -934,7 +870,6 @@ void SeqWidget::copyParamsFrom(SeqWidget *fromWidget)
     enableVelIn->setChecked(fromWidget->enableVelIn->isChecked());
     enableRestartByKbd->setChecked(fromWidget->enableRestartByKbd->isChecked());
     enableTrigByKbd->setChecked(fromWidget->enableTrigByKbd->isChecked());
-    enableLoop->setChecked(fromWidget->enableLoop->isChecked());
 
     tmp = fromWidget->chIn->currentIndex();
     chIn->setCurrentIndex(tmp);
@@ -952,6 +887,9 @@ void SeqWidget::copyParamsFrom(SeqWidget *fromWidget)
     tmp = fromWidget->sizeBox->currentIndex();
     sizeBox->setCurrentIndex(tmp);
     updateSize(tmp);
+    tmp = fromWidget->loopBox->currentIndex();
+    loopBox->setCurrentIndex(tmp);
+    updateLoop(tmp);
 
     tmp = fromWidget->velocity->value();
     updateVelocity(tmp);
@@ -966,6 +904,9 @@ void SeqWidget::copyParamsFrom(SeqWidget *fromWidget)
     for (int l1 = 0; l1 < midiWorker->customWave.count(); l1++) {
         midiWorker->muteMask.append(midiWorker->customWave.at(l1).muted);
     }
+    tmp = fromWidget->getLoopMarker();
+    midiWorker->loopMarker = tmp;
+    screen->setLoopMarker(tmp);
     midiControl->setCcList(fromWidget->midiControl->ccList);
     muteOut->setChecked(true);
     updateWaveForm(0);
@@ -975,3 +916,80 @@ QVector<Sample> SeqWidget::getCustomWave()
 {
     return midiWorker->customWave;
 }
+
+void SeqWidget::handleController(int ccnumber, int channel, int value)
+{
+    bool m;
+    int min, max, sval;
+    QVector<MidiCC> cclist= midiControl->ccList;
+
+    for (int l2 = 0; l2 < cclist.count(); l2++) {
+        min = cclist.at(l2).min;
+        max = cclist.at(l2).max;
+        if ((ccnumber == cclist.at(l2).ccnumber) &&
+            (channel == cclist.at(l2).channel)) {
+            switch (cclist.at(l2).ID) {
+                case 0: if (min == max) {
+                            if (value == max) {
+                                m = muteOut->isChecked();
+                                muteOut->setChecked(!m);
+                            }
+                        }
+                        else {
+                            if (value == max) {
+                                muteOut->setChecked(false);
+                            }
+                            if (value == min) {
+                                muteOut->setChecked(true);
+                            }
+                        }
+                break;
+
+                case 1:
+                        sval = min + ((double)value * (max - min) / 127);
+                        velocity->setValue(sval);
+                break;
+
+                case 2:
+                        sval = min + ((double)value * (max - min) / 127);
+                        notelength->setValue(sval);
+                break;
+
+                case 3: if (min == max) {
+                            if (value == max) {
+                                m = recordAction->isChecked();
+                                recordAction->setChecked(!m);
+                                return;
+                            }
+                        }
+                        else {
+                            if (value == max) {
+                                recordAction->setChecked(true);
+                            }
+                            if (value == min) {
+                                recordAction->setChecked(false);
+                            }
+                        }
+                break;
+                case 4:
+                        sval = min + ((double)value * (max - min) / 127);
+                        resBox->setCurrentIndex(sval);
+                        updateRes(sval);
+                break;
+                case 5:
+                        sval = min + ((double)value * (max - min) / 127);
+                        sizeBox->setCurrentIndex(sval);
+                        updateSize(sval);
+                break;
+                case 6:
+                        sval = min + ((double)value * (max - min) / 127);
+                        loopBox->setCurrentIndex(sval);
+                        updateLoop(sval);
+                break;
+
+                default:
+                break;
+            }
+        }
+    }
+}
